@@ -16,14 +16,17 @@ use POE::Component::IRC::Plugin qw(:ALL);
 use base qw(POE::Component::IRC);
 use vars qw($VERSION);
 
-$VERSION = '1.9';
+$VERSION = '2.40';
 
 # Event handlers for tracking the STATE. $self->{STATE} is used as our namespace.
 # u_irc() is used to create unique keys.
 
 # Make sure we have a clean STATE when we first join the network and if we inadvertently get disconnected
 sub S_001 {
-  delete $_[0]->{STATE};
+  my $self = shift;
+  delete $self->{STATE};
+  $self->{STATE}->{usermode} = '';
+  $self->yield( 'mode', $self->{RealNick} );
   return PCI_EAT_NONE;
 }
 
@@ -117,9 +120,20 @@ sub S_quit {
   my ($self,$irc) = splice @_, 0, 2;
   my $mapping = $irc->isupport('CASEMAPPING');
   my $nick = ( split /!/, ${ $_[0] } )[0];
+  my $msg = ${ $_[1] };
   push @{ $_[2] }, [ $self->nick_channels( $nick ) ];
   my $unick = u_irc $nick, $mapping;
 
+  # Check if it is a netsplit
+  if ( $msg ) {
+    SWITCH: {
+       my @args = split /\s/, $msg;
+       if ( @args != 2 ) {
+	 last SWITCH;
+       }
+    }
+  }
+
   if ( $unick eq u_irc ( $self->nick_name(), $mapping ) ) {
         delete $self->{STATE};
   } else {
@@ -137,6 +151,7 @@ sub S_kick {
   my $mapping = $irc->isupport('CASEMAPPING');
   my $channel = ${ $_[1] };
   my $nick = ${ $_[2] };
+  push @{ $_[$#_] }, $self->nick_long_form( $nick );
   my $unick = u_irc $nick, $mapping;
   my $uchan = u_irc $channel, $mapping;
 
@@ -196,7 +211,7 @@ sub S_chan_mode {
   pop @_;
   my $mode = ${ $_[2] };
   my $arg = ${ $_[3] };
-  return PCI_EAT_NONE unless $mynick = u_irc( $arg, $mapping ) and $mode =~ /\+[qoah]/;
+  return PCI_EAT_NONE unless $mode =~ /\+[qoah]/ and $mynick eq u_irc( $arg, $mapping );
   my $excepts = $irc->isupport('EXCEPTS');
   my $invex = $irc->isupport('INVEX');
   $irc->yield ( 'mode' => $channel => $excepts ) if $excepts;
@@ -204,6 +219,14 @@ sub S_chan_mode {
   return PCI_EAT_NONE;
 }
 
+sub S_221 {
+  my ($self,$irc) = splice @_, 0, 2;
+  my $mode = ${ $_[0] };
+  $mode =~ s/^\+//;
+  ($self->{STATE}->{usermode} = $mode ) =~ s/^\+//;
+  return PCI_EAT_NONE;
+}
+
 # Channel MODE
 sub S_mode {
   my ($self,$irc) = splice @_, 0, 2;
@@ -216,13 +239,14 @@ sub S_mode {
 
   # CHANMODES is [$list_mode, $always_arg, $arg_when_set, $no_arg]
   # A $list_mode always has an argument
-  my $statmodes = join '', keys %{ $irc->isupport('PREFIX') };
-  my $chanmodes = $irc->isupport('CHANMODES');
+  my $prefix = $irc->isupport('PREFIX') || { 'o', '@', 'v', '+' };
+  my $statmodes = join '', keys %{ $prefix };
+  my $chanmodes = $irc->isupport('CHANMODES') || [ qw(beI k l imnpstaqr) ];
   my $alwaysarg = join '', $statmodes,  @{ $chanmodes }[0 .. 1];
 
   # Do nothing if it is UMODE
   if ( $uchan ne u_irc ( $self->{RealNick}, $mapping ) ) {
-     my $parsed_mode = parse_mode_line( @modes );
+     my $parsed_mode = parse_mode_line( $prefix, $chanmodes, @modes );
      while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
         my $arg;
         $arg = shift ( @{ $parsed_mode->{args} } ) if ( $mode =~ /^(.[$alwaysarg]|\+[$chanmodes->[2]])/ );
@@ -295,6 +319,21 @@ sub S_mode {
         delete $self->{STATE}->{Chans}->{ $uchan }->{Mode};
      }
   }
+  else {
+     my $parsed_mode = parse_mode_line( @modes );
+     while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
+	if ( $mode =~ /^\+(.)/ ) {
+		my $flag = $1;
+		$self->{STATE}->{usermode} .= $flag unless $self->{STATE}->{usermode} =~ /$flag/;
+		next;
+	}
+        if ( $mode =~ /^-(.)/ ) {
+                my $flag = $1;
+		$self->{STATE}->{usermode} =~ s/$flag// if $self->{STATE}->{usermode} =~ /$flag/;
+		next;
+	}
+     }
+  }
   return PCI_EAT_NONE;
 }
 
@@ -328,7 +367,7 @@ sub S_352 {
   if ( $channel ne '*' ) {
     my $whatever = '';
     my $existing = $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } || '';    
-    my $prefix = $irc->isupport('PREFIX');
+    my $prefix = $irc->isupport('PREFIX') || { 'o', '@', 'v', '+' };
     foreach my $mode ( keys %{ $prefix } ) {
       $whatever .= $mode if ( $status =~ /\Q$prefix->{$mode}/ and $existing !~ /\Q$prefix->{$mode}/ );
     }
@@ -458,7 +497,7 @@ sub S_324 {
   my @args = @{ ${ $_[2] } };
   my $channel = shift @args;
   my $uchan = u_irc $channel, $mapping;
-  my $chanmodes = $irc->isupport('CHANMODES');
+  my $chanmodes = $irc->isupport('CHANMODES') || [ qw(beI k l imnpstaqr) ];
 
   my $parsed_mode = parse_mode_line( @args );
   while ( my $mode = shift ( @{ $parsed_mode->{modes} } ) ) {
@@ -514,13 +553,18 @@ sub S_333 {
 # Internal methods begin with '_'
 #
 
+sub umode {
+  my $self = shift;
+  return $self->{STATE}->{usermode};
+}
+
 sub _channel_sync {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
   my $sync = $_[1];
 
-  return 0 unless ( $self->_channel_exists($channel) and defined ( $self->{CHANNEL_SYNCH}->{ $channel } ) );
+  return 0 unless $self->_channel_exists($channel) and defined ( $self->{CHANNEL_SYNCH}->{ $channel } );
 
   $self->{CHANNEL_SYNCH}->{ $channel }->{ $sync } = 1 if $sync;
 
@@ -533,32 +577,34 @@ sub _channel_sync {
 
 sub _nick_exists {
   my $self = shift;
+  my $nick = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $nick = u_irc ( $_[0], $mapping ) || return 0;
+  $nick = u_irc $nick, $mapping;
   return 1 if defined $self->{STATE}->{Nicks}->{ $nick };
   return 0;
 }
 
 sub _channel_exists {
   my $self = shift;
+  my $channel = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  $channel = u_irc $channel, $mapping;
   return 1 if defined $self->{STATE}->{Chans}->{ $channel };
   return 0;
 }
 
 sub _nick_has_channel_mode {
   my $self = shift;
+  my $channel = shift || return;
+  my $nick = shift || return;
+  my $flag = shift || return;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
-  my $flag = ( split //, $_[2] )[0] || return 0;
+  $channel = u_irc $channel, $mapping;
+  $nick = u_irc $nick, $mapping;
+  $flag = ( split //, $flag )[0];
 
   return 0 unless $self->is_channel_member($channel,$nick);
-
-  if ( $self->{STATE}->{Nicks}->{ $nick }->{CHANS}->{ $channel } =~ /$flag/ ) {
-	return 1;
-  }
+  return 1 if $self->{STATE}->{Nicks}->{ $nick }->{CHANS}->{ $channel } =~ /$flag/;
   return 0;
 }
 
@@ -579,12 +625,7 @@ sub channels {
 
 sub nicks {
   my $self = shift;
-  my @result;
-
-  foreach my $nick ( keys %{ $self->{STATE}->{Nicks} } ) {
-	push @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick};
-  }
-  return @result;
+  return map { $self->{STATE}->{Nicks}->{$_}->{Nick} } keys %{ $self->{STATE}->{Nicks} };
 }
 
 sub nick_info {
@@ -595,13 +636,9 @@ sub nick_info {
   return unless $self->_nick_exists($nick);
 
   my $record = $self->{STATE}->{Nicks}->{ $nick };
-
   my %result = %{ $record };
-
   $result{Userhost} = $result{User} . '@' . $result{Host};
-
   delete $result{'CHANS'};
-
   return \%result;
 }
 
@@ -620,36 +657,27 @@ sub nick_long_form {
 sub nick_channels {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $nick = u_irc ( $_[0], $mapping ) || return;
+  my $nick = u_irc $_[0], $mapping || return;
   my @result;
 
   return unless $self->_nick_exists($nick);
-
-  foreach my $channel ( keys %{ $self->{STATE}->{Nicks}->{ $nick }->{CHANS} } ) {
-	push ( @result, $self->{STATE}->{Chans}->{ $channel }->{Name} );
-  }
-  return @result;
+  return map { $self->{STATE}->{Chans}->{$_}->{Name} } keys %{ $self->{STATE}->{Nicks}->{ $nick }->{CHANS} };
 }
 
 sub channel_list {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return;
+  my $channel = u_irc $_[0], $mapping || return;
   my @result;
 
   return unless $self->_channel_exists($channel);
-
-  foreach my $nick ( keys %{ $self->{STATE}->{Chans}->{ $channel }->{Nicks} } ) {
-	push @result, $self->{STATE}->{Nicks}->{ $nick }->{Nick};
-  }
-
-  return @result;
+  return map { $self->{STATE}->{Nicks}->{$_}->{Nick} } keys %{ $self->{STATE}->{Chans}->{ $channel }->{Nicks} };
 }
 
 sub is_operator {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $nick = u_irc ( $_[0], $mapping ) || return;
+  my $nick = u_irc $_[0], $mapping || return;
   return unless $self->_nick_exists($nick);
   return 1 if $self->{STATE}->{Nicks}->{ $nick }->{IRCop};
   return 0;
@@ -658,23 +686,21 @@ sub is_operator {
 sub is_channel_mode_set {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
   my $mode = ( split //, $_[1] )[0] || return 0;
 
   $mode =~ s/[^A-Za-z]//g;
 
   return unless $self->_channel_exists($channel) or $mode;
 
-  if ( defined ( $self->{STATE}->{Chans}->{ $channel }->{Mode} ) and $self->{STATE}->{Chans}->{ $channel }->{Mode} =~ /$mode/ ) {
-	return 1;
-  }
+  return 1 if defined ( $self->{STATE}->{Chans}->{ $channel }->{Mode} ) and $self->{STATE}->{Chans}->{ $channel }->{Mode} =~ /$mode/;
   return 0;
 }
 
 sub channel_limit {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
 
   return unless $self->_channel_exists($channel);
 
@@ -687,7 +713,7 @@ sub channel_limit {
 sub channel_key {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
 
   return unless $self->_channel_exists($channel);
 
@@ -713,8 +739,8 @@ sub channel_modes {
 sub is_channel_member {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return unless $self->_channel_exists($channel) and $self->_nick_exists($nick);
   return 1 if defined $self->{STATE}->{Chans}->{ $channel }->{Nicks}->{ $nick };
   return 0;
@@ -723,8 +749,8 @@ sub is_channel_member {
 sub is_channel_operator {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return 0 unless $self->_nick_has_channel_mode($channel,$nick,'o');
   return 1;
 }
@@ -732,8 +758,8 @@ sub is_channel_operator {
 sub has_channel_voice {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return 0 unless $self->_nick_has_channel_mode($channel,$nick,'v');
   return 1;
 }
@@ -741,8 +767,8 @@ sub has_channel_voice {
 sub is_channel_halfop {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return 0 unless $self->_nick_has_channel_mode($channel,$nick,'h');
   return 1;
 }
@@ -750,8 +776,8 @@ sub is_channel_halfop {
 sub is_channel_owner {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return 0 unless $self->_nick_has_channel_mode($channel,$nick,'q');
   return 1;
 }
@@ -759,8 +785,8 @@ sub is_channel_owner {
 sub is_channel_admin {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return 0;
-  my $nick = u_irc ( $_[1], $mapping ) || return 0;
+  my $channel = u_irc $_[0], $mapping || return 0;
+  my $nick = u_irc $_[1], $mapping || return 0;
   return 0 unless $self->_nick_has_channel_mode($channel,$nick,'a');
   return 1;
 }
@@ -768,7 +794,7 @@ sub is_channel_admin {
 sub ban_mask {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
   my $mask = parse_ban_mask ( $_[1] ) || return undef;
   my @result;
 
@@ -781,9 +807,7 @@ sub ban_mask {
   $mask =~ s/\\\?/[\x01-\xFF]{1,1}/g;
 
   foreach my $nick ( $self->channel_list($channel) ) {
-	if ( u_irc ( $self->nick_long_form($nick) ) =~ /^$mask$/ ) {
-		push ( @result, $nick );
-	}
+	push @result, $nick if u_irc ( $self->nick_long_form($nick) ) =~ /^$mask$/;
   }
 
   return @result;
@@ -792,7 +816,7 @@ sub ban_mask {
 sub channel_ban_list {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
   my %result;
 
   return undef unless $self->_channel_exists($channel);
@@ -807,7 +831,7 @@ sub channel_ban_list {
 sub channel_except_list {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
   my $excepts = $self->isupport('EXCEPTS');
   my %result;
 
@@ -823,7 +847,7 @@ sub channel_except_list {
 sub channel_invex_list {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
   my $invex = $self->isupport('INVEX');
   my %result;
 
@@ -839,7 +863,7 @@ sub channel_invex_list {
 sub channel_topic {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
   my %result;
 
   return undef unless $self->_channel_exists($channel);
@@ -854,8 +878,8 @@ sub channel_topic {
 sub nick_channel_modes {
   my $self = shift;
   my $mapping = $self->isupport('CASEMAPPING');
-  my $channel = u_irc ( $_[0], $mapping ) || return undef;
-  my $nick = u_irc ( $_[1], $mapping ) || return undef;
+  my $channel = u_irc $_[0], $mapping || return undef;
+  my $nick = u_irc $_[1], $mapping || return undef;
 
   return undef unless $self->is_channel_member($channel, $nick);
 
@@ -976,6 +1000,10 @@ All of the L<POE::Component::IRC> methods are supported, plus the following:
 
 =over
 
+=item umode
+
+Takes no parameters. Returns the current user mode set for the bot.
+
 =item channels
 
 Takes no parameters. Returns a hashref, keyed on channel name and whether the bot is operator, halfop or 
@@ -1141,13 +1169,17 @@ ARG2 contains an arrayref of channel names that are common to the quitting clien
 
 ARG2 contains an arrayref of channel names that are common to the nick changing client and the component.
 
+=item irc_kick
+
+Additional parameter ARG4 contains the full nick!user@host of the kicked individual.
+
 =back
 
 =head1 CAVEATS
 
 The component gathers information by registering for 'irc_quit', 'irc_nick', 'irc_join', 'irc_part', 'irc_mode', 'irc_kick' and
 various numeric replies. When the component is asked to join a channel, when it joins it will issue a 'WHO #channel' and a 'MODE #channel'. These will solicit between them the numerics, 'irc_352' and 'irc_324'.
-You may want to ignore these. When someones joins a channel the bot is on, it issues a 'WHO nick'.
+You may want to ignore these. When someone joins a channel the bot is on, it issues a 'WHO nick'.
 
 Currently, whenever the component sees a topic or channel list change, it will use time() for the SetAt value and the full address of the user who set it for the SetBy value. When an ircd gives us it's record 
 of such changes, it will use it's own time (obviously) and may only give us the nickname of the user, rather than their full address. Thus, if our time() and the ircd's time do not match, or the ircd uses the
